package com.zfire.jiasm.syncdata.service.impl;

import com.zfire.jiasm.syncdata.base.BaseService;
import com.zfire.jiasm.syncdata.dao.mapper.InstallDownloadTaskListMapper;
import com.zfire.jiasm.syncdata.dao.mapper.InstallXzydDownloadMapper;
import com.zfire.jiasm.syncdata.data.DetailDownloadData;
import com.zfire.jiasm.syncdata.parameter.OrderPar;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.service.ItfTblAzAssignXzydDownloadService;
import com.zfire.jiasm.syncdata.plus.service.OrderBaseService;
import com.zfire.jiasm.syncdata.plus.service.OrderInformationService;
import com.zfire.jiasm.syncdata.response.WebsitFWS;
import com.zfire.jiasm.syncdata.service.DetailDownloadService;
import com.zfire.jiasm.syncdata.service.PublicService;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class DetailDownloadServiceImpl implements DetailDownloadService {
	@Autowired
	private JdbcTemplate jdbcTemplate;
	@Resource
	ItfTblAzAssignXzydDownloadService itfTblAzAssignXzydDownloadService;
	@Resource
	InstallXzydDownloadMapper installXzydDownloadMapper;
	@Resource
	InstallDownloadTaskListMapper installDownloadTaskListMapper;
	@Resource
	OrderInformationService orderInformationService;
	@Resource
	PublicService publicService;
	@Resource
	BaseService baseService;
	@Resource
	OrderBaseService orderBaseService;
	

	@Override
	public List<Map<String, Object>> getTaskTypeInstall() {
		List<Map<String, Object>> result = jdbcTemplate.queryForList("SELECT * from "
				+" itf_gree_to_fa_download_task_type  "
				+ " where download_task_type not in (92,31) "
				);
		return result;
	}

	@Override
	public List<Map<String, Object>> getTaskTypeRepair() {
		List<Map<String, Object>> result = jdbcTemplate.queryForList("SELECT * from "
				+" itf_gree_to_fa_download_task_type  "
				+ " where download_task_type in (92,31) "
				);
		return result;
	}
	
	@Override
	public List<Map<String, Object>> getTaskDataInstall(int downloadTaskType) {
		List<Map<String, Object>> result = jdbcTemplate.queryForList("SELECT a.*, "
				+ " c.download_task_type_name "
				+ " FROM itf_gree_to_fa_download_task_list a,itf_gree_to_fa_download_task_type c "
				+ " where a.download_task_type=c.download_task_type " 
				+ " and c.download_task_type not in (92,31) and c.download_task_type=? "
				+ " and a.download_status=0 and (download_times is null or download_times<=5) order by a.id limit 1000 ",new Object[] {downloadTaskType});
		return result;
	}
	
	@Override
	public List<Map<String, Object>> getTaskDataRepair(int downloadTaskType) {
		List<Map<String, Object>> result = jdbcTemplate.queryForList("SELECT a.*, "
				+ " c.download_task_type_name "
				+ " FROM itf_gree_to_fa_download_task_list a,itf_gree_to_fa_download_task_type c "
				+ " where a.download_task_type=c.download_task_type " 
				+ " and c.download_task_type in (92,31) and c.download_task_type=? "
				+ " and a.download_status=0 and (download_times is null or download_times<=5) order by a.id limit 1000 ",new Object[] {downloadTaskType});
		return result;
	}

	public void insertAData(String downloadTaskNo, String destTabName, String destTabColList,
			Object[] arySrcTabColValue) throws Exception {

		StringBuffer mark = new StringBuffer();
		for (int i = 0; i < arySrcTabColValue.length; i++) {
			if (mark.length() > 0) {
				mark.append(",");
			}
			mark.append("?");
		}

		String sql = "insert into " + destTabName + " (download_task_no," + destTabColList + ") values (?," + mark
				+ ") ";

		Object[] insertValue = new Object[arySrcTabColValue.length + 1];
		insertValue[0] = downloadTaskNo;
		System.arraycopy(arySrcTabColValue, 0, insertValue, 1, arySrcTabColValue.length);

		jdbcTemplate.update(sql, insertValue);

	}

	@Override
	public List<Map<String, Object>> getTaskTables(int downloadTaskType) {
		List<Map<String, Object>> result = jdbcTemplate.queryForList(
				"SELECT * from itf_gree_to_fa_download_task_rule where download_task_type=? ",
				new Object[] { downloadTaskType });
		return result;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void insertData(String downloadTaskNo, List<DetailDownloadData> detailDownloadDatas,
						   int downloadTaskType, String pgguid)
			throws Exception {

		Map<String, List<SysDict>> sysDictRefDLMap = new HashMap<>();

		// 提取品牌信息
		Brand brand = baseService.getBrand();
		if (brand == null) {
			return;
		}

		sysDictRefDLMap = publicService.getStringListMapXqLb();

		// 写入下载表
		for (DetailDownloadData detailDownloadData : detailDownloadDatas) {
			insertAData(downloadTaskNo, detailDownloadData.getDestTabName(), detailDownloadData.getDestTabColList(),
					detailDownloadData.getArySrcTabColValue());
		}

		// 写入本地表
		if (downloadTaskType == 2206){
			// itf_tbl_az_assign_xzyd_download  TL_SynDataFrom_tbl_az_assign_xzyd
			processInformatino(downloadTaskNo, pgguid, sysDictRefDLMap, brand);
		}
	}

	public void processInformatino(String downloadTaskNo, String pgguid, Map<String, List<SysDict>> sysDictRefDLMap, Brand brand) {
		// 提取数据
		List<ItfTblAzAssignXzydDownload> zdmx = itfTblAzAssignXzydDownloadService.lambdaQuery()
				.eq(ItfTblAzAssignXzydDownload::getDownloadTaskNo, downloadTaskNo)
				.eq(ItfTblAzAssignXzydDownload::getSynStatus,0).list();

		if (!CollectionUtils.isEmpty(zdmx)){
			// 取最大的syn_req_id 用于判断是否为最后一条记录
			OrderPar req = new OrderPar();
			req.setOrderNo(downloadTaskNo);
			ItfTblAzAssignXzydDownload maxzd = installXzydDownloadMapper.getSynReqId(req);
			OrderInformation info = new OrderInformation();
			WebsitFWS sh = new WebsitFWS();
			for (ItfTblAzAssignXzydDownload item:zdmx){
				// 存在更新，不存在插入 order_information
				info = orderInformationService.lambdaQuery()
						.eq(OrderInformation::getId,item.getId()).one();

				// 工单ID
				OrderBase orderBase = orderBaseService.lambdaQuery()
						.eq(OrderBase::getPgguid, pgguid).one();

				if (info !=null){
					// 次数，与是否已读不相同时修改，其他不修改 这里只能做是否已读的处理
					if (!info.getReadStatus().equals(item.getYdbz())){
						upInformation(sysDictRefDLMap, brand, info, item);
					}
				} else {
					addInformation(sysDictRefDLMap, brand, item, orderBase);
				}

				// 最后一条记录时更新主表的子信息字段
				if (maxzd.getSynReqId().equals(item.getSynReqId()) && !orderBase.getIsChildInfo().equals("YES")){
					orderBase.setIsChildInfo("YES");
					orderBase.updateById();
				}
			}
		}
		// 修改同步状态
		upSynStatus(downloadTaskNo);
	}

	public void upInformation(@NotNull Map<String, List<SysDict>> sysDictRefDLMap, Brand brand, OrderInformation info, ItfTblAzAssignXzydDownload item) {
		WebsitFWS sh;
		info.setAskCategoryId(item.getXzyqlb());
		info.setAskCategoryName(sysDictRefDLMap.get(item.getXzyqlb()).get(0).getDictValue());
		info.setRemark(item.getXzyq());
		if (item.getYdbz().equals(1)){
			info.setReadStatus(item.getYdbz());
		} else {
			info.setReadStatus(0);
		}
		info.setReadName(item.getYdrmc());
		info.setReadTime(item.getYdsj());
		// 网点信息单独处理
		if (item.getYdbz().equals(1)){
			sh = publicService.WebsitToSH(item.getYdwd(), brand.getBrandId());
		} else {
			sh = publicService.WebsitToSH(item.getWdno(), brand.getBrandId());
		}
		info.setWebsitId(sh.getWdId());
		info.setWebsitName(sh.getWdmc());
		info.updateById();
	}

	public void addInformation(Map<String, List<SysDict>> sysDictRefDLMap, Brand brand,
								ItfTblAzAssignXzydDownload item, OrderBase orderBase) {
		WebsitFWS sh;
		OrderInformation info = new OrderInformation();
		info.setId(item.getId());
		info.setAskCategoryId(item.getXzyqlb());
		info.setAskCategoryName(sysDictRefDLMap.get(item.getXzyqlb()).get(0).getDictValue());
		info.setOrderBaseId(orderBase.getId());
		info.setRemark(item.getXzyq());
		info.setCreateBy(item.getCreatedBy());
		info.setCreateTime(item.getCreatedDate());
		info.setUpdateBy(item.getLastModifiedBy());
		info.setUpdateTime(item.getLastModifiedDate());
		// 网点信息单独处理
		if (item.getYdbz().equals(1)){
			sh = publicService.WebsitToSH(item.getYdwd(), brand.getBrandId());
		} else {
			sh = publicService.WebsitToSH(item.getWdno(), brand.getBrandId());
		}
		info.setWebsitId(sh.getWdId());
		info.setWebsitName(sh.getWdmc());
		if (item.getYdbz().equals(1)){
			info.setReadStatus(item.getYdbz());
		} else {
			info.setReadStatus(0);
		}
		info.setWebsitName(sh.getWdmc());
		info.setReadTime(item.getYdsj());
		info.insert();
	}

	public void upSynStatus(String downloadTaskNo) {
		OrderPar req = new OrderPar();
		req.setOrderNo(downloadTaskNo);
		installXzydDownloadMapper.updateSynStatus(req);
		installDownloadTaskListMapper.updateSynStatus(req);
	}

	@Override
	public void updateError(String downloadTaskNo, Integer downloadTimes, String errMessage) {
		if (downloadTimes == null) {
			downloadTimes = 1;
		} else {
			downloadTimes++;
		}
		jdbcTemplate.update(
				"UPDATE itf_gree_to_fa_download_task_list set download_err_msg=?,download_times=? where download_task_no=?",
				new Object[] { errMessage, downloadTimes, downloadTaskNo });
	}

}
